<?php
require_once("ews/ExchangeServices.php");
require_once("include.php");

class SubscriptionServer extends ExchangeServicesServerImpl
{
    public function SubscriptionServer($soapServer) {
        parent::ExchangeServicesServerImpl($soapServer);
    }
    
    public function SendNotification(EWS_SendNotificationResponseType $result) {
        $subscriptionStatus = EWS_SendNotificationResultType::SubscriptionStatus_Unsubscribe;
        try
        {
            if($result->ResponseMessages->SendNotificationResponseMessage) {
                /** @var EWS_SendNotificationResponseMessageType*/ $response = $result->ResponseMessages->SendNotificationResponseMessage;
                if($response->ResponseClass == EWS_SendNotificationResponseMessageType::ResponseClass_Success) {
                    if($response->Notification && $response->Notification->SubscriptionId != "") {
                        $updateAvailability = false;
                        if($response->Notification->FreeBusyChangedEvent) {
                            $freeBusyEvents = $response->Notification->FreeBusyChangedEvent;
                            if(!is_array($freeBusyEvents)) {
                                $freeBusyEvents = array($freeBusyEvents);
                            }
                            foreach($freeBusyEvents as /** @var EWS_BaseObjectChangedEventType*/ $event) {
                                if($event->ItemId && $event->ItemId->Id != "") {
                                    // there has been an calendar item update, so update the availability status to retrieve new items the next time
                                    $updateAvailability = true;
                                    break;
                                }
                            }
                        }
                        if(!$updateAvailability && $response->Notification->ModifiedEvent) {
                            $modifiedEvents = $response->Notification->ModifiedEvent;
                            if(!is_array($modifiedEvents)) {
                                $modifiedEvents = array($modifiedEvents);
                            }
                            foreach($modifiedEvents as /** @var EWS_BaseObjectChangedEventType*/ $event) {
                                if($event->ItemId && $event->ItemId->Id != "") {
                                    // there has been an calendar item update, so update the availability status to retrieve new items the next time
                                    $updateAvailability = true;
                                    break;
                                }
                            }
                        }
                        $exchangeDb = get_exchange_db();
                        $query = "UPDATE users SET 
                                    subscription_status=date_trunc('second', localtimestamp)";
                        if($updateAvailability) {
                            $query .= ", availability_status=NULL";
                        }
                        $query .=   " WHERE subscription_id='".pg_escape_string($response->Notification->SubscriptionId)."'";
                        $dbResult = $exchangeDb->run($query);
                        if($dbResult === false) {
                            throw new PbxException("Updating user subscription failed", pg_last_error($exchangeDb->db), Alarms::alExchangeUpdateUserSubscription2);
                        }
                        else if(pg_affected_rows($dbResult) == 1){
                            $subscriptionStatus = EWS_SendNotificationResultType::SubscriptionStatus_OK;
                        }
                    }
                }
            }
        }
        catch(PbxException $e) {
            send_alarm($e->getMessage(), $e->details, $e->errorCode, false, ExchangeMethods::getAlarmServer(), "", false, "Exchange");
        }
        $return = new EWS_SendNotificationResultType();
        $return->SubscriptionStatus = $subscriptionStatus;
        if($subscriptionStatus == EWS_SendNotificationResultType::SubscriptionStatus_OK) {
            clear_alarm(Alarms::alExchangeUpdateUserSubscription2, ExchangeMethods::getAlarmServer(), "Exchange");
        }
        return $return;
    }
}

/*$postData = '<?xml version="1.0" encoding="utf-8"?>
<soap11:Envelope xmlns:soap11="http://schemas.xmlsoap.org/soap/envelope/">
  <soap11:Header>
    <t:RequestServerVersion xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages" xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types" Version="Exchange2010_SP1"/>
  </soap11:Header>
  <soap11:Body>
    <m:SendNotification xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types" xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
      <m:ResponseMessages>
        <m:SendNotificationResponseMessage ResponseClass="Success">
          <m:ResponseCode>NoError</m:ResponseCode>
          <m:Notification>
            <t:SubscriptionId>HQBpbm5vLWV4LTIwMTAuaW5ub3ZhcGhvbmUuc2lmaRAAAADqOTLifykFQYcNZ7IeRcmHFhVdwsC+zgg=</t:SubscriptionId>
            <t:PreviousWatermark>AQAAANGQOvWYor5Ojo7L/zMXAfWSbasDAAAAAAA=</t:PreviousWatermark>
            <t:MoreEvents>false</t:MoreEvents>
            <t:StatusEvent>
              <t:Watermark>AQAAANGQOvWYor5Ojo7L/zMXAfXWbasDAAAAAAE=</t:Watermark>
            </t:StatusEvent>
          </m:Notification>
        </m:SendNotificationResponseMessage>
      </m:ResponseMessages>
    </m:SendNotification>
  </soap11:Body>
</soap11:Envelope>
';*/

$RAW_POST_DATA = file_get_contents("php://input");
$server = new ExchangeServicesServer("SubscriptionServer");
$server->handle($RAW_POST_DATA);

//$result = $soapServer->service(/*$HTTP_RAW_POST_DATA*/$postData);
//var_dump($result);

/*$resp = '<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope">
  <soap:Body>
    <SendNotificationResult xmlns="http://schemas.microsoft.com/exchange/services/2006/messages">
<!--      <SubscriptionStatus>OK</SubscriptionStatus>-->
      <SubscriptionStatus>Unsubscribe</SubscriptionStatus>
    </SendNotificationResult>
  </soap:Body>
</soap:Envelope>';
echo $resp;*/

global $mainSession;
$mainSession->destroy();
?>